from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="TrackerPostMetaAttr")


@attr.s(auto_attribs=True)
class TrackerPostMetaAttr(OSIDBModel):
    """ """

    bz_id: Union[Unset, str] = UNSET
    owner: Union[Unset, str] = UNSET
    qe_owner: Union[Unset, str] = UNSET
    ps_component: Union[Unset, str] = UNSET
    ps_module: Union[Unset, str] = UNSET
    ps_update_stream: Union[Unset, str] = UNSET
    resolution: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bz_id = self.bz_id
        owner = self.owner
        qe_owner = self.qe_owner
        ps_component = self.ps_component
        ps_module = self.ps_module
        ps_update_stream = self.ps_update_stream
        resolution = self.resolution
        status = self.status

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(bz_id, Unset):
            field_dict["bz_id"] = bz_id
        if not isinstance(owner, Unset):
            field_dict["owner"] = owner
        if not isinstance(qe_owner, Unset):
            field_dict["qe_owner"] = qe_owner
        if not isinstance(ps_component, Unset):
            field_dict["ps_component"] = ps_component
        if not isinstance(ps_module, Unset):
            field_dict["ps_module"] = ps_module
        if not isinstance(ps_update_stream, Unset):
            field_dict["ps_update_stream"] = ps_update_stream
        if not isinstance(resolution, Unset):
            field_dict["resolution"] = resolution
        if not isinstance(status, Unset):
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bz_id = d.pop("bz_id", UNSET)

        owner = d.pop("owner", UNSET)

        qe_owner = d.pop("qe_owner", UNSET)

        ps_component = d.pop("ps_component", UNSET)

        ps_module = d.pop("ps_module", UNSET)

        ps_update_stream = d.pop("ps_update_stream", UNSET)

        resolution = d.pop("resolution", UNSET)

        status = d.pop("status", UNSET)

        tracker_post_meta_attr = cls(
            bz_id=bz_id,
            owner=owner,
            qe_owner=qe_owner,
            ps_component=ps_component,
            ps_module=ps_module,
            ps_update_stream=ps_update_stream,
            resolution=resolution,
            status=status,
        )

        tracker_post_meta_attr.additional_properties = d
        return tracker_post_meta_attr

    @staticmethod
    def get_fields():
        return {
            "bz_id": str,
            "owner": str,
            "qe_owner": str,
            "ps_component": str,
            "ps_module": str,
            "ps_update_stream": str,
            "resolution": str,
            "status": str,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional

import requests

from ...client import AuthenticatedClient
from ...models.affect_post import AffectPost
from ...models.osidb_api_v1_affects_bulk_create_response_200 import (
    OsidbApiV1AffectsBulkCreateResponse200,
)
from ...types import UNSET, Response, Unset

QUERY_PARAMS = {}
REQUEST_BODY_TYPE = List[AffectPost]


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    multipart_data: List[AffectPost],
    json_body: List[AffectPost],
) -> Dict[str, Any]:
    url = "{}/osidb/api/v1/affects/bulk".format(
        client.base_url,
    )

    headers: Dict[str, Any] = client.get_headers()

    json_json_body: List[Dict[str, Any]] = UNSET
    if not isinstance(json_body, Unset):
        json_json_body = []
        for json_body_item_data in json_body:
            json_body_item: Dict[str, Any] = UNSET
            if not isinstance(json_body_item_data, Unset):
                json_body_item = json_body_item_data.to_dict()

            json_json_body.append(json_body_item)

    multipart_multipart_data: List[Dict[str, Any]] = UNSET
    if not isinstance(multipart_data, Unset):
        multipart_multipart_data = []
        for multipart_data_item_data in multipart_data:
            multipart_data_item: Dict[str, Any] = UNSET
            if not isinstance(multipart_data_item_data, Unset):
                multipart_data_item = multipart_data_item_data.to_dict()

            multipart_multipart_data.append(multipart_data_item)

    return {
        "url": url,
        "headers": headers,
        "json": json_json_body,
    }


def _parse_response(
    *, response: requests.Response
) -> Optional[OsidbApiV1AffectsBulkCreateResponse200]:
    if response.status_code == 200:
        _response_200 = response.json()
        response_200: OsidbApiV1AffectsBulkCreateResponse200
        if isinstance(_response_200, Unset):
            response_200 = UNSET
        else:
            response_200 = OsidbApiV1AffectsBulkCreateResponse200.from_dict(
                _response_200
            )

        return response_200
    return None


def _build_response(
    *, response: requests.Response
) -> Response[OsidbApiV1AffectsBulkCreateResponse200]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    multipart_data: List[AffectPost],
    json_body: List[AffectPost],
) -> Response[OsidbApiV1AffectsBulkCreateResponse200]:
    kwargs = _get_kwargs(
        client=client,
        multipart_data=multipart_data,
        json_body=json_body,
    )

    response = requests.post(
        verify=client.verify_ssl,
        auth=client.auth,
        timeout=client.timeout,
        **kwargs,
    )
    response.raise_for_status()

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    multipart_data: List[AffectPost],
    json_body: List[AffectPost],
) -> Optional[OsidbApiV1AffectsBulkCreateResponse200]:
    """Bulk create endpoint. Expects a list of dict Affect objects."""

    return sync_detailed(
        client=client,
        multipart_data=multipart_data,
        json_body=json_body,
    ).parsed


async def async_detailed(
    *,
    client: AuthenticatedClient,
    multipart_data: List[AffectPost],
    json_body: List[AffectPost],
) -> Response[OsidbApiV1AffectsBulkCreateResponse200]:
    kwargs = _get_kwargs(
        client=client,
        multipart_data=multipart_data,
        json_body=json_body,
    )

    async with client.get_async_session().post(
        verify_ssl=client.verify_ssl, raise_for_status=True, **kwargs
    ) as response:
        content = await response.read()
        resp = requests.Response()
        resp.status_code = response.status
        resp._content = content

    return _build_response(response=resp)


async def async_(
    *,
    client: AuthenticatedClient,
    multipart_data: List[AffectPost],
    json_body: List[AffectPost],
) -> Optional[OsidbApiV1AffectsBulkCreateResponse200]:
    """Bulk create endpoint. Expects a list of dict Affect objects."""

    return (
        await async_detailed(
            client=client,
            multipart_data=multipart_data,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.affect import Affect
from ..models.module_component import ModuleComponent
from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="TrackerSuggestion")


@attr.s(auto_attribs=True)
class TrackerSuggestion(OSIDBModel):
    """ """

    modules_components: List[ModuleComponent]
    not_applicable: List[Affect]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        modules_components: List[Dict[str, Any]] = UNSET
        if not isinstance(self.modules_components, Unset):
            modules_components = []
            for modules_components_item_data in self.modules_components:
                modules_components_item: Dict[str, Any] = UNSET
                if not isinstance(modules_components_item_data, Unset):
                    modules_components_item = modules_components_item_data.to_dict()

                modules_components.append(modules_components_item)

        not_applicable: List[Dict[str, Any]] = UNSET
        if not isinstance(self.not_applicable, Unset):
            not_applicable = []
            for not_applicable_item_data in self.not_applicable:
                not_applicable_item: Dict[str, Any] = UNSET
                if not isinstance(not_applicable_item_data, Unset):
                    not_applicable_item = not_applicable_item_data.to_dict()

                not_applicable.append(not_applicable_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(modules_components, Unset):
            field_dict["modules_components"] = modules_components
        if not isinstance(not_applicable, Unset):
            field_dict["not_applicable"] = not_applicable

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        modules_components = []
        _modules_components = d.pop("modules_components", UNSET)
        if _modules_components is UNSET:
            modules_components = UNSET
        else:
            for modules_components_item_data in _modules_components or []:
                _modules_components_item = modules_components_item_data
                modules_components_item: ModuleComponent
                if isinstance(_modules_components_item, Unset):
                    modules_components_item = UNSET
                else:
                    modules_components_item = ModuleComponent.from_dict(
                        _modules_components_item
                    )

                modules_components.append(modules_components_item)

        not_applicable = []
        _not_applicable = d.pop("not_applicable", UNSET)
        if _not_applicable is UNSET:
            not_applicable = UNSET
        else:
            for not_applicable_item_data in _not_applicable or []:
                _not_applicable_item = not_applicable_item_data
                not_applicable_item: Affect
                if isinstance(_not_applicable_item, Unset):
                    not_applicable_item = UNSET
                else:
                    not_applicable_item = Affect.from_dict(_not_applicable_item)

                not_applicable.append(not_applicable_item)

        tracker_suggestion = cls(
            modules_components=modules_components,
            not_applicable=not_applicable,
        )

        tracker_suggestion.additional_properties = d
        return tracker_suggestion

    @staticmethod
    def get_fields():
        return {
            "modules_components": List[ModuleComponent],
            "not_applicable": List[Affect],
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="FlawMetaAttr")


@attr.s(auto_attribs=True)
class FlawMetaAttr(OSIDBModel):
    """ """

    acknowledgments: Union[Unset, str] = UNSET
    acks_not_needed: Union[Unset, str] = UNSET
    affects: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    bz_datascore: Union[Unset, str] = UNSET
    bz_id: Union[Unset, str] = UNSET
    checklists: Union[Unset, str] = UNSET
    classification: Union[Unset, str] = UNSET
    cwe: Union[Unset, str] = UNSET
    depends_on: Union[Unset, str] = UNSET
    impact: Union[Unset, str] = UNSET
    jira_trackers: Union[Unset, str] = UNSET
    mitigate: Union[Unset, str] = UNSET
    mitigation: Union[Unset, str] = UNSET
    public: Union[Unset, str] = UNSET
    references: Union[Unset, str] = UNSET
    related_cves: Union[Unset, str] = UNSET
    reported: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET
    statement: Union[Unset, str] = UNSET
    task_owner: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        acknowledgments = self.acknowledgments
        acks_not_needed = self.acks_not_needed
        affects = self.affects
        alias = self.alias
        bz_datascore = self.bz_datascore
        bz_id = self.bz_id
        checklists = self.checklists
        classification = self.classification
        cwe = self.cwe
        depends_on = self.depends_on
        impact = self.impact
        jira_trackers = self.jira_trackers
        mitigate = self.mitigate
        mitigation = self.mitigation
        public = self.public
        references = self.references
        related_cves = self.related_cves
        reported = self.reported
        source = self.source
        statement = self.statement
        task_owner = self.task_owner

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(acknowledgments, Unset):
            field_dict["acknowledgments"] = acknowledgments
        if not isinstance(acks_not_needed, Unset):
            field_dict["acks_not_needed"] = acks_not_needed
        if not isinstance(affects, Unset):
            field_dict["affects"] = affects
        if not isinstance(alias, Unset):
            field_dict["alias"] = alias
        if not isinstance(bz_datascore, Unset):
            field_dict["bz_datascore"] = bz_datascore
        if not isinstance(bz_id, Unset):
            field_dict["bz_id"] = bz_id
        if not isinstance(checklists, Unset):
            field_dict["checklists"] = checklists
        if not isinstance(classification, Unset):
            field_dict["classification"] = classification
        if not isinstance(cwe, Unset):
            field_dict["cwe"] = cwe
        if not isinstance(depends_on, Unset):
            field_dict["depends_on"] = depends_on
        if not isinstance(impact, Unset):
            field_dict["impact"] = impact
        if not isinstance(jira_trackers, Unset):
            field_dict["jira_trackers"] = jira_trackers
        if not isinstance(mitigate, Unset):
            field_dict["mitigate"] = mitigate
        if not isinstance(mitigation, Unset):
            field_dict["mitigation"] = mitigation
        if not isinstance(public, Unset):
            field_dict["public"] = public
        if not isinstance(references, Unset):
            field_dict["references"] = references
        if not isinstance(related_cves, Unset):
            field_dict["related_cves"] = related_cves
        if not isinstance(reported, Unset):
            field_dict["reported"] = reported
        if not isinstance(source, Unset):
            field_dict["source"] = source
        if not isinstance(statement, Unset):
            field_dict["statement"] = statement
        if not isinstance(task_owner, Unset):
            field_dict["task_owner"] = task_owner

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        acknowledgments = d.pop("acknowledgments", UNSET)

        acks_not_needed = d.pop("acks_not_needed", UNSET)

        affects = d.pop("affects", UNSET)

        alias = d.pop("alias", UNSET)

        bz_datascore = d.pop("bz_datascore", UNSET)

        bz_id = d.pop("bz_id", UNSET)

        checklists = d.pop("checklists", UNSET)

        classification = d.pop("classification", UNSET)

        cwe = d.pop("cwe", UNSET)

        depends_on = d.pop("depends_on", UNSET)

        impact = d.pop("impact", UNSET)

        jira_trackers = d.pop("jira_trackers", UNSET)

        mitigate = d.pop("mitigate", UNSET)

        mitigation = d.pop("mitigation", UNSET)

        public = d.pop("public", UNSET)

        references = d.pop("references", UNSET)

        related_cves = d.pop("related_cves", UNSET)

        reported = d.pop("reported", UNSET)

        source = d.pop("source", UNSET)

        statement = d.pop("statement", UNSET)

        task_owner = d.pop("task_owner", UNSET)

        flaw_meta_attr = cls(
            acknowledgments=acknowledgments,
            acks_not_needed=acks_not_needed,
            affects=affects,
            alias=alias,
            bz_datascore=bz_datascore,
            bz_id=bz_id,
            checklists=checklists,
            classification=classification,
            cwe=cwe,
            depends_on=depends_on,
            impact=impact,
            jira_trackers=jira_trackers,
            mitigate=mitigate,
            mitigation=mitigation,
            public=public,
            references=references,
            related_cves=related_cves,
            reported=reported,
            source=source,
            statement=statement,
            task_owner=task_owner,
        )

        flaw_meta_attr.additional_properties = d
        return flaw_meta_attr

    @staticmethod
    def get_fields():
        return {
            "acknowledgments": str,
            "acks_not_needed": str,
            "affects": str,
            "alias": str,
            "bz_datascore": str,
            "bz_id": str,
            "checklists": str,
            "classification": str,
            "cwe": str,
            "depends_on": str,
            "impact": str,
            "jira_trackers": str,
            "mitigate": str,
            "mitigation": str,
            "public": str,
            "references": str,
            "related_cves": str,
            "reported": str,
            "source": str,
            "statement": str,
            "task_owner": str,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="TokenVerify")


@attr.s(auto_attribs=True)
class TokenVerify(OSIDBModel):
    """ """

    token: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        token = self.token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(token, Unset):
            field_dict["token"] = token

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        token = (
            self.token if self.token is UNSET else (None, str(self.token), "text/plain")
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                key: (None, str(value), "text/plain")
                for key, value in self.additional_properties.items()
            }
        )
        if not isinstance(token, Unset):
            field_dict["token"] = token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        token = d.pop("token", UNSET)

        token_verify = cls(
            token=token,
        )

        token_verify.additional_properties = d
        return token_verify

    @staticmethod
    def get_fields():
        return {
            "token": str,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.affect import Affect
from ..models.ps_stream_selection import PsStreamSelection
from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="ModuleComponent")


@attr.s(auto_attribs=True)
class ModuleComponent(OSIDBModel):
    """ """

    ps_module: str
    ps_component: str
    streams: List[PsStreamSelection]
    selected: bool
    affect: Affect
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ps_module = self.ps_module
        ps_component = self.ps_component
        streams: List[Dict[str, Any]] = UNSET
        if not isinstance(self.streams, Unset):
            streams = []
            for streams_item_data in self.streams:
                streams_item: Dict[str, Any] = UNSET
                if not isinstance(streams_item_data, Unset):
                    streams_item = streams_item_data.to_dict()

                streams.append(streams_item)

        selected = self.selected
        affect: Dict[str, Any] = UNSET
        if not isinstance(self.affect, Unset):
            affect = self.affect.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(ps_module, Unset):
            field_dict["ps_module"] = ps_module
        if not isinstance(ps_component, Unset):
            field_dict["ps_component"] = ps_component
        if not isinstance(streams, Unset):
            field_dict["streams"] = streams
        if not isinstance(selected, Unset):
            field_dict["selected"] = selected
        if not isinstance(affect, Unset):
            field_dict["affect"] = affect

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ps_module = d.pop("ps_module", UNSET)

        ps_component = d.pop("ps_component", UNSET)

        streams = []
        _streams = d.pop("streams", UNSET)
        if _streams is UNSET:
            streams = UNSET
        else:
            for streams_item_data in _streams or []:
                _streams_item = streams_item_data
                streams_item: PsStreamSelection
                if isinstance(_streams_item, Unset):
                    streams_item = UNSET
                else:
                    streams_item = PsStreamSelection.from_dict(_streams_item)

                streams.append(streams_item)

        selected = d.pop("selected", UNSET)

        _affect = d.pop("affect", UNSET)
        affect: Affect
        if isinstance(_affect, Unset):
            affect = UNSET
        else:
            affect = Affect.from_dict(_affect)

        module_component = cls(
            ps_module=ps_module,
            ps_component=ps_component,
            streams=streams,
            selected=selected,
            affect=affect,
        )

        module_component.additional_properties = d
        return module_component

    @staticmethod
    def get_fields():
        return {
            "ps_module": str,
            "ps_component": str,
            "streams": List[PsStreamSelection],
            "selected": bool,
            "affect": Affect,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MajorIncidentStateEnum(str, Enum):
    REQUESTED = "REQUESTED"
    REJECTED = "REJECTED"
    APPROVED = "APPROVED"
    CISA_APPROVED = "CISA_APPROVED"
    INVALID = "INVALID"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class RequiresCveDescriptionEnum(str, Enum):
    REQUESTED = "REQUESTED"
    APPROVED = "APPROVED"
    REJECTED = "REJECTED"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="AffectBulkPutMetaAttr")


@attr.s(auto_attribs=True)
class AffectBulkPutMetaAttr(OSIDBModel):
    """ """

    affectedness: Union[Unset, str] = UNSET
    component: Union[Unset, str] = UNSET
    impact: Union[Unset, str] = UNSET
    module_name: Union[Unset, str] = UNSET
    module_stream: Union[Unset, str] = UNSET
    ps_component: Union[Unset, str] = UNSET
    ps_module: Union[Unset, str] = UNSET
    ps_product: Union[Unset, str] = UNSET
    resolution: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        affectedness = self.affectedness
        component = self.component
        impact = self.impact
        module_name = self.module_name
        module_stream = self.module_stream
        ps_component = self.ps_component
        ps_module = self.ps_module
        ps_product = self.ps_product
        resolution = self.resolution

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(affectedness, Unset):
            field_dict["affectedness"] = affectedness
        if not isinstance(component, Unset):
            field_dict["component"] = component
        if not isinstance(impact, Unset):
            field_dict["impact"] = impact
        if not isinstance(module_name, Unset):
            field_dict["module_name"] = module_name
        if not isinstance(module_stream, Unset):
            field_dict["module_stream"] = module_stream
        if not isinstance(ps_component, Unset):
            field_dict["ps_component"] = ps_component
        if not isinstance(ps_module, Unset):
            field_dict["ps_module"] = ps_module
        if not isinstance(ps_product, Unset):
            field_dict["ps_product"] = ps_product
        if not isinstance(resolution, Unset):
            field_dict["resolution"] = resolution

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        affectedness = d.pop("affectedness", UNSET)

        component = d.pop("component", UNSET)

        impact = d.pop("impact", UNSET)

        module_name = d.pop("module_name", UNSET)

        module_stream = d.pop("module_stream", UNSET)

        ps_component = d.pop("ps_component", UNSET)

        ps_module = d.pop("ps_module", UNSET)

        ps_product = d.pop("ps_product", UNSET)

        resolution = d.pop("resolution", UNSET)

        affect_bulk_put_meta_attr = cls(
            affectedness=affectedness,
            component=component,
            impact=impact,
            module_name=module_name,
            module_stream=module_stream,
            ps_component=ps_component,
            ps_module=ps_module,
            ps_product=ps_product,
            resolution=resolution,
        )

        affect_bulk_put_meta_attr.additional_properties = d
        return affect_bulk_put_meta_attr

    @staticmethod
    def get_fields():
        return {
            "affectedness": str,
            "component": str,
            "impact": str,
            "module_name": str,
            "module_stream": str,
            "ps_component": str,
            "ps_module": str,
            "ps_product": str,
            "resolution": str,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="Erratum")


@attr.s(auto_attribs=True)
class Erratum(OSIDBModel):
    """Erratum serializer"""

    et_id: int
    advisory_name: str
    created_dt: datetime.datetime
    updated_dt: datetime.datetime
    shipped_dt: Optional[datetime.datetime]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        et_id = self.et_id
        advisory_name = self.advisory_name
        created_dt: str = UNSET
        if not isinstance(self.created_dt, Unset):
            created_dt = self.created_dt.isoformat()

        updated_dt: str = UNSET
        if not isinstance(self.updated_dt, Unset):
            updated_dt = self.updated_dt.isoformat()

        shipped_dt: Optional[str] = UNSET
        if not isinstance(self.shipped_dt, Unset):
            shipped_dt = self.shipped_dt.isoformat() if self.shipped_dt else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(et_id, Unset):
            field_dict["et_id"] = et_id
        if not isinstance(advisory_name, Unset):
            field_dict["advisory_name"] = advisory_name
        if not isinstance(created_dt, Unset):
            field_dict["created_dt"] = created_dt
        if not isinstance(updated_dt, Unset):
            field_dict["updated_dt"] = updated_dt
        if not isinstance(shipped_dt, Unset):
            field_dict["shipped_dt"] = shipped_dt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        et_id = d.pop("et_id", UNSET)

        advisory_name = d.pop("advisory_name", UNSET)

        _created_dt = d.pop("created_dt", UNSET)
        created_dt: datetime.datetime
        if isinstance(_created_dt, Unset):
            created_dt = UNSET
        else:
            created_dt = isoparse(_created_dt)

        _updated_dt = d.pop("updated_dt", UNSET)
        updated_dt: datetime.datetime
        if isinstance(_updated_dt, Unset):
            updated_dt = UNSET
        else:
            updated_dt = isoparse(_updated_dt)

        _shipped_dt = d.pop("shipped_dt", UNSET)
        shipped_dt: Optional[datetime.datetime]
        if _shipped_dt is None:
            shipped_dt = None
        elif isinstance(_shipped_dt, Unset):
            shipped_dt = UNSET
        else:
            shipped_dt = isoparse(_shipped_dt)

        erratum = cls(
            et_id=et_id,
            advisory_name=advisory_name,
            created_dt=created_dt,
            updated_dt=updated_dt,
            shipped_dt=shipped_dt,
        )

        erratum.additional_properties = d
        return erratum

    @staticmethod
    def get_fields():
        return {
            "et_id": int,
            "advisory_name": str,
            "created_dt": datetime.datetime,
            "updated_dt": datetime.datetime,
            "shipped_dt": datetime.datetime,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import requests

from ...client import AuthenticatedClient
from ...models.osidb_api_v1_schema_retrieve_format import OsidbApiV1SchemaRetrieveFormat
from ...models.osidb_api_v1_schema_retrieve_lang import OsidbApiV1SchemaRetrieveLang
from ...models.osidb_api_v1_schema_retrieve_response_200 import (
    OsidbApiV1SchemaRetrieveResponse200,
)
from ...types import UNSET, Response, Unset

QUERY_PARAMS = {
    "format": OsidbApiV1SchemaRetrieveFormat,
    "lang": OsidbApiV1SchemaRetrieveLang,
}


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, None, OsidbApiV1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, None, OsidbApiV1SchemaRetrieveLang] = UNSET,
) -> Dict[str, Any]:
    url = "{}/osidb/api/v1/schema/".format(
        client.base_url,
    )

    headers: Dict[str, Any] = client.get_headers()

    json_format_: Union[Unset, None, str] = UNSET
    if not isinstance(format_, Unset):

        json_format_ = (
            OsidbApiV1SchemaRetrieveFormat(format_).value if format_ else None
        )

    json_lang: Union[Unset, None, str] = UNSET
    if not isinstance(lang, Unset):

        json_lang = OsidbApiV1SchemaRetrieveLang(lang).value if lang else None

    params: Dict[str, Any] = {
        "format": json_format_,
        "lang": json_lang,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "params": params,
    }


def _parse_response(
    *, response: requests.Response
) -> Optional[OsidbApiV1SchemaRetrieveResponse200]:
    if response.status_code == 200:
        _response_200 = response.json()
        response_200: OsidbApiV1SchemaRetrieveResponse200
        if isinstance(_response_200, Unset):
            response_200 = UNSET
        else:
            response_200 = OsidbApiV1SchemaRetrieveResponse200.from_dict(_response_200)

        return response_200
    return None


def _build_response(
    *, response: requests.Response
) -> Response[OsidbApiV1SchemaRetrieveResponse200]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, None, OsidbApiV1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, None, OsidbApiV1SchemaRetrieveLang] = UNSET,
) -> Response[OsidbApiV1SchemaRetrieveResponse200]:
    kwargs = _get_kwargs(
        client=client,
        format_=format_,
        lang=lang,
    )

    response = requests.get(
        verify=client.verify_ssl,
        auth=client.auth,
        timeout=client.timeout,
        **kwargs,
    )
    response.raise_for_status()

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, None, OsidbApiV1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, None, OsidbApiV1SchemaRetrieveLang] = UNSET,
) -> Optional[OsidbApiV1SchemaRetrieveResponse200]:
    """OpenApi3 schema for this API. Format can be selected via content negotiation.

    - YAML: application/vnd.oai.openapi
    - JSON: application/vnd.oai.openapi+json"""

    return sync_detailed(
        client=client,
        format_=format_,
        lang=lang,
    ).parsed


async def async_detailed(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, None, OsidbApiV1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, None, OsidbApiV1SchemaRetrieveLang] = UNSET,
) -> Response[OsidbApiV1SchemaRetrieveResponse200]:
    kwargs = _get_kwargs(
        client=client,
        format_=format_,
        lang=lang,
    )

    async with client.get_async_session().get(
        verify_ssl=client.verify_ssl, raise_for_status=True, **kwargs
    ) as response:
        content = await response.read()
        resp = requests.Response()
        resp.status_code = response.status
        resp._content = content

    return _build_response(response=resp)


async def async_(
    *,
    client: AuthenticatedClient,
    format_: Union[Unset, None, OsidbApiV1SchemaRetrieveFormat] = UNSET,
    lang: Union[Unset, None, OsidbApiV1SchemaRetrieveLang] = UNSET,
) -> Optional[OsidbApiV1SchemaRetrieveResponse200]:
    """OpenApi3 schema for this API. Format can be selected via content negotiation.

    - YAML: application/vnd.oai.openapi
    - JSON: application/vnd.oai.openapi+json"""

    return (
        await async_detailed(
            client=client,
            format_=format_,
            lang=lang,
        )
    ).parsed



"""
osidb-bindings constants
"""


from typing import List

OSIDB_API_VERSION: str = "v1"
OSIDB_BINDINGS_VERSION: str = "4.1.1"
OSIDB_BINDINGS_USERAGENT: str = f"osidb-bindings-{OSIDB_BINDINGS_VERSION}"
OSIDB_BINDINGS_API_PATH: str = ".bindings.python_client.api.osidb"
OSIDB_BINDINGS_PLACEHOLDER_FIELD: str = (
    f'__placeholder_field{OSIDB_BINDINGS_VERSION.replace(".","")}'
)

DEFAULT_LIMIT: int = 50

# all available session operations
ALL_SESSION_OPERATIONS: List[str] = (
    "retrieve",
    "update",
    "list",
    "create",
    "destroy",
    "search",
)



from enum import Enum


class OsidbApiV1FlawsListReferencesType(str, Enum):
    ARTICLE = "ARTICLE"
    EXTERNAL = "EXTERNAL"
    SOURCE = "SOURCE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, OSIDBModel, Unset

T = TypeVar("T", bound="OsidbWhoamiRetrieveResponse200Profile")


@attr.s(auto_attribs=True)
class OsidbWhoamiRetrieveResponse200Profile(OSIDBModel):
    """ """

    bz_user_id: Union[Unset, str] = UNSET
    jira_user_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bz_user_id = self.bz_user_id
        jira_user_id = self.jira_user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        if not isinstance(bz_user_id, Unset):
            field_dict["bz_user_id"] = bz_user_id
        if not isinstance(jira_user_id, Unset):
            field_dict["jira_user_id"] = jira_user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bz_user_id = d.pop("bz_user_id", UNSET)

        jira_user_id = d.pop("jira_user_id", UNSET)

        osidb_whoami_retrieve_response_200_profile = cls(
            bz_user_id=bz_user_id,
            jira_user_id=jira_user_id,
        )

        osidb_whoami_retrieve_response_200_profile.additional_properties = d
        return osidb_whoami_retrieve_response_200_profile

    @staticmethod
    def get_fields():
        return {
            "bz_user_id": str,
            "jira_user_id": str,
        }

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OsidbApiV1FlawsListImpact(str, Enum):
    VALUE_0 = ""
    CRITICAL = "CRITICAL"
    IMPORTANT = "IMPORTANT"
    LOW = "LOW"
    MODERATE = "MODERATE"

    def __str__(self) -> str:
        return str(self.value)
